/* 
JFU	20170826
DonP 20080624

Attribution and License:

 In accordance with the licensing information and guidelines for the 
 OBJECTIVE DEVELOPMENT USB driver code for microcontrollers (which this 
 is based on) I am including the following:

    This is source code for my "Haunted, Mystery USB Device".
	Copyright (C) 2008  Donald Papp (donp@imakeprojects.com)	

 (More specifically, the portions changed and written entirely by me 
 - as well as the circuitboard design to go with this code, which is not 
 included here - are copyrighted by me.  However, they are open sourced under
 the GPL.)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.


History:
	This source code began as the program for the Thermostat based on AVR USB driver
	by Christian Starkjohann.  (c) 2006 by OBJECTIVE DEVELOPMENT Software GmbH.

	That code served as the basis for Garrett Mace to make his Stealth USB CapsLocker
	(http://www.macetech.com/blog/?q=node/46) which served as partial inspiration to me.

	Learning from that code (and using it as a base) this code is now the 
	source for the HAUNTED, MYSTERY USB DEVICE (working name).  
	(http://www.imakeprojects.com/Projects/haunted-usb/)


My Version:		1.0

June 24, 2008
		- Altered the timing and delays of the injected keyboard events.
		- Added functionality for an external LED which is flickered in a 
		malfunctioning-looking way.
		- Added events (other than capslock toggling) and code to select between them.
		- Added prng() function for use as an RNG.

July 10, 2008
		- Modified code so that CAPS is toggled 2x right off the bat everytime so you
		can see it's working when you plug it in.  Does this via a global variable 
		that is set and never unset.
		As long as 'sendInitialCapsLock' is >0 then the random delay and selection of key 
		to send is overridden.  The var is decremented once the capslock keypress is built 
		into a report.  We want caps lock toggled twice shortly after startup so that
		you can see it's working. (Either blinks on, or blinks off, depending on the 
		initial state.)

Aug 26, 2017 Jindra Fucik
		- Extend to 16 keys
		- keys are stored in table
		- make random more random (every sequence starts different)

*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#include "usbdrv.h"
#include "oddebug.h"


/* ------------------------------------------------------------------------- */

static uchar    reportBuffer[2];    /* buffer for HID reports */
static uchar    idleRate;           /* in 4 ms units */
static uchar    reportCount;		/* current report */
static unsigned int	TimerDelay;		/* counter for delay period */

// DonP
static uchar	sendInitialCapsLock;// Whether to override normal delay and random key in
									// reports.
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};

//const PROGMEM // this table should be in progmem, but GCC not work well with it.
const uchar RandomKeys[32] = { /*List of used random keys*/
	//modifier, key
	0x00, 0x39,	//0 0x39 Caps lock
	0x00, 0x49,	//1 0x49 Insert
	0x00, 0x2A,	//2 0x2A Backspace
	0x00, 0x2C,	//3 0x2C Space
	0x00, 0x53,	//4 0x53 Numlock	
	0x00, 0x2B,	//5 0x2B Tab 	
	0x00, 0x4C, //6 0x4C Delete	
	0x08, 0x0F,	//7 Left GUI + L (Lock windows)
	0x04, 0x2B, //8 Left Alt + Tab
	0x00, 0x47, //9 Scrol Lock
	0x00, 0x50,	//A Left Arrow
	0x00, 0x39,	//B 0x39 Caps lock
	0x00, 0x2A,	//C 0x2A Backspace
	0x00, 0x2C,	//D 0x2C Space
	0x00, 0x53,	//E 0x53 Numlock	
	0x00, 0x2B	//F 0x2B Tab

	/* 0x00, 0x04,	//0 a	keys for debug only
	0x00, 0x05,	//1 b
	0x00, 0x06,	//2 c
	0x00, 0x07,	//3 d
	0x00, 0x08,	//4 e
	0x00, 0x09,	//5 f
	0x00, 0x0A, //6 g
	0x00, 0x0B,	//7 h
	0x00, 0x0C, //8 i
	0x00, 0x0D, //9 j
	0x00, 0x0E,	//A k
	0x00, 0x0F,	//B l
	0x00, 0x10,	//C m
	0x00, 0x11,	//D n
	0x00, 0x12,	//E o
	0x00, 0x13	//F p */

};


/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */

static void buildReport(void)
{
	uchar   key = 0;
	uchar 	modifier = 0;
	uchar x;
	//uchar *xptr;

    if(reportCount == 0)
	{
		// we have 16 keys 0-F, but LSB is added for modifier/key
		x = rand() & 0x1E;

		modifier = RandomKeys[x++];
		key = RandomKeys[x];

		if( sendInitialCapsLock > 0 )
		{

			// We always enter this function 1 time right off the bat, so ignore it
			// if that's the case (actually, make it an empty report).
			if( sendInitialCapsLock!=3 )
			{
				modifier = 0x00;
				key = 0x39;		// CapsLock
			}
			else
			{
				modifier = 0x00;
				key = 0x00;
			}
			
			sendInitialCapsLock--;
		}

	}


	reportCount++;
    reportBuffer[0] = modifier;
    reportBuffer[1] = key;
}

static void timerPoll(void)
{
	static unsigned int timerCnt;

    if(TIFR & (1 << TOV1))
	{
        TIFR = (1 << TOV1); /* clear overflow */

		/* check for end of pseudorandom delay */
        if( ++timerCnt >= TimerDelay )	
		{
			TimerDelay = 2835 + rand();		/* 1/63s * 63 * 30 + 0...32767 = 30sec to 9min10sec*/
			//TimerDelay = 63; // 1 sec
			//TimerDelay = 315; // DonP Test, 5 seconds constant 

			// DonP - ensure delay is overridden to ~1 second if this flag is set.
			// Actually only if it's greater than 1 otherwise the next event AFTER
			// the initial caps lock toggle is 1 second after the last toggle.
			// In other words if the flag is 1 the NEXT delay is what we want to be normal.
			// (Not the one after the flag is 0.)
			if( sendInitialCapsLock > 1)
				TimerDelay = 40;

			timerCnt = 0;
			reportCount = 0; /* start report */
        }
    }
}

/* ------------------------------------------------------------------------- */

static void timerInit(void)
{
    TCCR1 = 0x0b;           /* select clock: 16.5M/1k -> overflow rate = 16.5M/256k = 62.94 Hz */
}


/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            buildReport();
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}

/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}
/*
Note: This calibration algorithm may try OSCCAL values of up to 192 even if
the optimum value is far below 192. It may therefore exceed the allowed clock
frequency of the CPU in low voltage designs!
You may replace this search algorithm with any other algorithm you like if
you have additional constraints such as a maximum CPU clock.
For version 5.x RC oscillators (those with a split range of 2x128 steps, e.g.
ATTiny25, ATTiny45, ATTiny85), it may be useful to search for the optimum in
both regions.
*/

void    usbEventResetReady(void)
{
    calibrateOscillator();
    eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}






/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
uchar   calibrationValue;
uint16_t resetCnt;

    calibrationValue = eeprom_read_byte(0); /* calibration value from last time */
    resetCnt = eeprom_read_word(1); /* reset counter */
	if (0 == ++resetCnt) {resetCnt=1;}
	eeprom_update_word(1,resetCnt);
    if(calibrationValue != 0xff){
        OSCCAL = calibrationValue;
    }
    odDebugInit();
    usbDeviceDisconnect();
    for(i=0;i<20;i++){  /* 300 ms disconnect */
        _delay_ms(15);
    }
    usbDeviceConnect();
    timerInit();

    wdt_enable(WDTO_1S);
	//TimerDelay = 630; /* initial 10 second delay */
	TimerDelay = 200; /* initial ~3 second delay */


	// DonP - 20080710
	// By setting the 'sendInitialCapsLock' override flag to 4, after powerup
	// it will wait 3 seconds (initial time delay) then toggle the CAPS lock twice.
	// Non-zero 'sendInitialCapsLock' means:
	// 1. Override key selection to be alternately the 'CapsLock' event, or an empty report.
	// 2. Override random interval delay (after initial 3 second delay) to be 1 second.
	// 3. Decrement the 'sendInitialCapsLock' value by 1.

	sendInitialCapsLock = 3; // DonP - set flag to override key selection this many times+1.
	// (+1 because buildreport runs at powerup once and we want to ignore that.)

    usbInit();
	srand(resetCnt);
    sei();
    for(;;){    /* main event loop */
        wdt_reset();
        usbPoll();

		/* A USB keypress cycle is defined as a scancode being present in a report, and
		then absent from a later report. To press and release the Caps Lock key, instead of
		holding it down, we need to send the report with the Caps Lock scancode and
		then an empty report. */

        if(usbInterruptIsReady() && reportCount < 2){ /* we can send another key */
            buildReport();
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
        	}
        timerPoll();
    }
    return 0;
}
